#!/bin/bash

##########################################################################
# 
# This file is part of FORCE - Framework for Operational Radiometric 
# Correction for Environmental monitoring.
# 
# Copyright (C) 2013-2018 David Frantz
# 
# FORCE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# FORCE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with FORCE.  If not, see <http://www.gnu.org/licenses/>.
# 
##########################################################################


EXPECTED_ARGS=1

# if wrong number of input args, stop
if [ $# -ne $EXPECTED_ARGS ]; then
  echo "Usage: `basename $0` tiled-archive"
  exit
fi

NOW=$PWD

INP=$(readlink -f $1)
OUT=$INP/mosaic
LIST=$OUT/list.txt


# input dir exists?
if [ ! -d $INP ]; then
  echo $INP "does not exist"
  exit
fi

# output dir exists?
if [ ! -d $OUT ]; then
  mkdir $OUT
  if [ ! -d $OUT ]; then
    echo "creating directory failed."
    exit
  fi
fi

cd $OUT


FILES=$(find .. \( -name '*.dat' -o -name '*.tif' \) -exec basename {} \; | sort | uniq | xargs)

# for each possible output product
for prd in $FILES; do 

  echo "mosaicking" $prd
  
  ONAME=${prd/.dat/.vrt}
  ONAME=${ONAME/.tif/.vrt}

  # file list (relative to $OUT)
  find .. -name $prd > $LIST

  # file list exists?
  if [ ! -f $LIST ]; then
    echo "could not create file listing."
    exit
  fi

  # number of chips
  N=$(wc -l $LIST | cut -d " " -f 1)

  # nodata value
  FIRST=$(head -1 $LIST)
  NODATA=$(gdalinfo $FIRST | grep 'NoData Value' | head -1 | cut -d '=' -f 2)


  # build vrt
  if [ $N -gt 0 ]; then
    echo $N "chips found".
    gdalbuildvrt -q -srcnodata $NODATA -vrtnodata $NODATA -input_file_list $LIST $OUT"/"$ONAME
    sed -i.tmp 's/relativeToVRT="0"/relativeToVRT="1"/g' $OUT"/"$ONAME
    chmod --reference $OUT"/"$ONAME".tmp" $OUT"/"$ONAME
    rm $OUT"/"$ONAME".tmp"
  else
    echo "no chip found."
  fi

  # delete list
  rm $LIST
  if [ -f $LIST ]; then
    echo "deleting file listing failed."
    exit
  fi

done

cd $PWD

