#!/bin/bash

##########################################################################
# 
# This file is part of FORCE - Framework for Operational Radiometric 
# Correction for Environmental monitoring.
# 
# Copyright (C) 2013-2018 David Frantz
# 
# FORCE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# FORCE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with FORCE.  If not, see <http://www.gnu.org/licenses/>.
# 
##########################################################################


EXPECTED_ARGS=3

# if wrong number of input args, stop
if [ $# -ne $EXPECTED_ARGS ]
then
  echo "Usage: `basename $0` L2-archive QUICKLOOK-archive maxval"
  exit
fi

INP=$1
OUT=$2
MAX=$3


# input dir exists?
if [ ! -d $INP ]; then
  echo $INP "does not exist"
  exit
fi

# output dir exists?
if [ ! -d $OUT ]; then
  echo $OUT "does not exist"
  exit
fi


for f in $(find $INP \( -name '*BOA.dat' -o -name '*BOA.tif' \)); do

  FNAME=$(basename $f)
  TNAME=$(echo $FNAME | sed 's/\.[a-z]\{3\}$/\.jpg/')

  TILE=$OUT"/"$(dirname $f | sed 's/.*\///')
  ONAME=$TILE"/"$TNAME


  if [ -f $ONAME ]; then
    continue;
  fi


  # create target directory if it doesn't exist
  if [ ! -w $TILE ]; then
    mkdir $TILE
    if [ ! -w $TILE ]; then
      echo "creating directory failed"
      exit
    fi
  fi


  # create quicklook
  gdal_translate -of 'JPEG' -ot 'Byte' -b 3 -b 2 -b 1 -outsize 256 256 -scale 0 $MAX -q $f $ONAME

  if [ ! -f $ONAME ]; then
    echo "creating quicklook failed"
    exit
  fi

  rm $TILE/*.xml

done

