#!/bin/bash

##########################################################################
# 
# This file is part of FORCE - Framework for Operational Radiometric 
# Correction for Environmental monitoring.
# 
# Copyright (C) 2013-2018 David Frantz
# 
# FORCE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# FORCE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with FORCE.  If not, see <http://www.gnu.org/licenses/>.
# 
##########################################################################



EXPECTED_ARGS=3
MAXIMUM_ARGS=4
EXIT=0
DOTILE=1

# if wrong number of input args, stop
if [ $# -ne $EXPECTED_ARGS ]; then
  if [ $# -ne $MAXIMUM_ARGS ]; then
    EXIT=1
  else
    if [ $4 == "--no-tile" ]; then
      DOTILE=0
    else
      EXIT=1
    fi
  fi
fi

if [ $EXIT -ne 0 ]; then
  echo "Usage: `basename $0` L3-archive QUICKLOOK-archive maxval [--no-tile]"
  exit
fi

INP=$1
OUT=$2
MAX=$3

# append mosaic subfolder to  input dir
if [ $DOTILE -eq 0 ]; then
  INP=$INP"/mosaic"
fi

# input dir exists?
if [ ! -d $INP ]; then
  echo $INP "does not exist"
  exit
fi

# output dir exists?
if [ ! -d $OUT ]; then
  echo $OUT "does not exist"
  exit
fi


for f in $(find $INP \( -name '*BAP*.dat' -o -name '*BAP*.vrt' -o -name '*BAP*.tif' \)); do

  FNAME=$(basename $f)
  TNAME=$(echo $FNAME | sed 's/\.[a-z]\{3\}$/\.jpg/')

  if [ $DOTILE -eq 1 ]; then
    TILE=$OUT"/"$(dirname $f | sed 's/.*\///')
  else
    TILE=$OUT"/mosaic"
  fi
  ONAME=$TILE"/"$TNAME


  if [ -f $ONAME ]; then
    continue;
  fi


  # create target directory if it doesn't exist
  if [ ! -w $TILE ]; then
    mkdir $TILE
    if [ ! -w $TILE ]; then
      echo "creating directory failed"
      exit
    fi
  fi

  # size of quicklook
  if [ $DOTILE -eq 1 ]; then
    NX=256
    NY=256
  else
    NX="20%"
    NY="20%"
  fi

  # create quicklook
  gdal_translate -of 'JPEG' -ot 'Byte' -b 3 -b 2 -b 1 -outsize $NX $NY -scale 0 $MAX -q $f $ONAME

  if [ ! -f $ONAME ]; then
    echo "creating quicklook failed"
    exit
  fi

  rm $TILE/*.xml

done

