/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

This file is part of FORCE - Framework for Operational Radiometric 
Correction for Environmental monitoring.

Copyright (C) 2013-2018 David Frantz

FORCE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FORCE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FORCE.  If not, see <http://www.gnu.org/licenses/>.

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Global variables
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/


#ifndef GLOBAL_H
#define GLOBAL_H

#ifdef __cplusplus
extern "C" {
#endif


/** global variables
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

extern par_t  par;  // processing parameters
extern meta_t meta; // metadata
extern atc_t  atc;  // atmospheric correction variables
extern dir_t  dir;  // directory listings

extern int _NX_;    // number of samples
extern int _NY_;    // number of lines
extern int _NC_;    // number of cells

extern int _NB_;    // number of input  reflectance bands
extern int _NO_;    // number of output reflectance bands
extern int _ND_;    // number of bands in output stack
extern int _NP_;    // number of output products

extern bands_t _B_;  // pointers to specific bands in reflectance input stack

extern const double _R2D_CONV_; // radians to degree conversion
extern const double _D2R_CONV_; // degree to radians conversion
extern const double _M_PI2_;    // 2*pi

extern const float _RS_; // reflectance scale factor
extern const float _TS_; // temperature scale factor

extern const int2s _NODATA_;    // nodata value, reflectance
extern const int   _QAIOFF_;    // nodata value, quality assurance inf.
extern const int2s _TSAOFF_;    // nodata value, time series

extern struct tm *timeinfo;     // time struct

#ifdef __cplusplus
}
#endif

#endif
