/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

This file is part of FORCE - Framework for Operational Radiometric 
Correction for Environmental monitoring.

Copyright (C) 2013-2018 David Frantz

FORCE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FORCE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FORCE.  If not, see <http://www.gnu.org/licenses/>.

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
This program generates a parameter file skeleton, which may be used to
parameterize the FORCE Clear-Sky-Observation stats module
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/


#include "force.h"


int main( int argc, char *argv[] ){
FILE *fp;
char fname[MAX_STR_LEN];

  if (argc != 2){ printf("usage: %s dir\n", argv[0]); exit(1);}

  sprintf(fname, "%s/%s", argv[1], "cso-skeleton.prm");
  if ((fp = fopen(fname, "w")) == NULL){
    printf("Unable to open output file!\n"); exit(1); }

  fprintf(fp, "++PARAM_CSO_START++\n\n");

  fprintf(fp, "# DIRECTORIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Level-2 datapool (input)\nDIR_LEVEL2 = NULL\n");
  fprintf(fp, "# CSO datapool (output)\nDIR_CSO = NULL\n");
  fprintf(fp, "# White list of input/output tiles\nFILE_TILE  = NULL\n\n");

  fprintf(fp, "# SENSOR WHITE LIST\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Select observations from following sensors\n# See documentation for valid keys\n");
  fprintf(fp, "SENSORS = LND04 LND05 LND07 LND08 SEN2A SEN2B\n\n");

  fprintf(fp, "# QAI SCREENING\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# This list controls, which QAI flags are masked out.\n# See documentation for valid keys\n");
  fprintf(fp, "SCREEN_QAI = NODATA CLOUD_OPAQUE CLOUD_BUFFER CLOUD_CIRRUS CLOUD_SHADOW SNOW SUBZERO SATURATION\n\n");

  fprintf(fp, "# SPATIAL PROPERTIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Composite extent in tile coordinates\n");
  fprintf(fp, "X_TILE_MIN = 0\n");
  fprintf(fp, "Y_TILE_MIN = 0\n");
  fprintf(fp, "X_TILE_MAX = 9999\n");
  fprintf(fp, "Y_TILE_MAX = 9999\n");
  fprintf(fp, "# Spatial resolution of composite\n# Image decimation/replication is taken care of\n");
  fprintf(fp, "RESOLUTION = 30\n\n");
  
  fprintf(fp, "# TEMPORAL PROPERTIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Year range\nYEAR_MIN = 1900\nYEAR_MAX = 2100\n");
  fprintf(fp, "# Intervall in months\nMONTH_STEP = 3\n\n");

  fprintf(fp, "# MISCELLANEOUS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Number of CPUs for parallel processing\nNUM_CPU = 1\n\n");

  fprintf(fp, "# OUTPUT\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Output format (ENVI or GTiff)\nOUTPUT_FORMAT = GTiff\n");
  fprintf(fp, "# Number of clear-sky obs.\nOUTPUT_NUM = TRUE\n");
  fprintf(fp, "# Average of dt of clear-sky obs.\nOUTPUT_AVG = TRUE\n");
  fprintf(fp, "# Standard dev. of dt of clear-sky obs.\nOUTPUT_STD = TRUE\n");
  fprintf(fp, "# Minimum of dt of clear-sky obs.\nOUTPUT_MIN = FALSE\n");
  fprintf(fp, "# Maximum of dt of clear-sky obs.\nOUTPUT_MAX = FALSE\n");
  fprintf(fp, "# Range of dt of clear-sky obs.\nOUTPUT_RNG = FALSE\n");
  fprintf(fp, "# Skewness of dt of clear-sky obs.\nOUTPUT_SKW = FALSE\n");
  fprintf(fp, "# Kurtosis of dt of clear-sky obs.\nOUTPUT_KRT = FALSE\n");
  fprintf(fp, "# 25%%-Quantiles of dt of clear-sky obs.\nOUTPUT_Q25 = FALSE\n");
  fprintf(fp, "# 50%%-Quantiles of dt of clear-sky obs.\nOUTPUT_Q50 = FALSE\n");
  fprintf(fp, "# 75%%-Quantiles of dt of clear-sky obs.\nOUTPUT_Q75 = FALSE\n");
  fprintf(fp, "# Interquantile range of dt of clear-sky obs.\nOUTPUT_IQR = FALSE\n\n");

  fprintf(fp, "++PARAM_CSO_END++\n");

  fclose(fp);

  return SUCCESS;
}

