/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

This file is part of FORCE - Framework for Operational Radiometric 
Correction for Environmental monitoring.

Copyright (C) 2013-2018 David Frantz

FORCE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FORCE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FORCE.  If not, see <http://www.gnu.org/licenses/>.

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
This program generates a parameter file skeleton, which may be used to
parameterize the FORCE Level 2 ImproPhe component
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/


#include "force.h"


int main( int argc, char *argv[] ){
FILE *fp;
char fname[MAX_STR_LEN];

  if (argc != 2){ printf("usage: %s dir\n", argv[0]); exit(1);}

  sprintf(fname, "%s/%s", argv[1], "l2imp-skeleton.prm");
  if ((fp = fopen(fname, "w")) == NULL){
    printf("Unable to open output file!\n"); exit(1); }

  fprintf(fp, "++PARAM_L2IMP_START++\n\n");

  fprintf(fp, "# DIRECTORIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Level-2 datapool (input)\nDIR_LEVEL2 = NULL\n");
  fprintf(fp, "# White list of input/output tiles\nFILE_TILE  = NULL\n\n");

  fprintf(fp, "# SENSOR WHITE LIST\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Improve the spatial resolution of following sensors\n# See documentation for valid keys\n");
  fprintf(fp, "SENSORS_MR = LND07 LND08\n");
  fprintf(fp, "# Use following sensors for the improvement\n# See documentation for valid keys\n");
  fprintf(fp, "SENSORS_HR = sen2a sen2b\n\n");

  fprintf(fp, "# QAI SCREENING\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# This list controls, which QAI flags are masked out.\n# See documentation for valid keys\n");
  fprintf(fp, "SCREEN_QAI = NODATA CLOUD_OPAQUE CLOUD_BUFFER CLOUD_CIRRUS CLOUD_SHADOW SNOW SUBZERO SATURATION\n\n");

  fprintf(fp, "# SPATIAL PROPERTIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Analysis extent in tile coordinates\n");
  fprintf(fp, "X_TILE_MIN = 0\n");
  fprintf(fp, "Y_TILE_MIN = 0\n");
  fprintf(fp, "X_TILE_MAX = 9999\n");
  fprintf(fp, "Y_TILE_MAX = 9999\n");
  fprintf(fp, "# Spatial resolution of the improved data\n");
  fprintf(fp, "# Resolution should be >= HR data and < MR data\n");
  fprintf(fp, "# Image decimation/replication is taken care of\n");
  fprintf(fp, "RESOLUTION = 10\n");
  fprintf(fp, "# Use an approx. Point Spread Function when reducing spatial resolution\n");
  fprintf(fp, "REDUCE_PSF = FALSE\n\n");

  fprintf(fp, "# TEMPORAL PROPERTIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Year to be predicted\nYEAR = 2016\n");
  fprintf(fp, "# Breakpoints (DOY) for the seasonal windows\nSEASONAL_WINDOW = 1 91 181 271 365\n\n");

  fprintf(fp, "# PREDICTION PARAMETERS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Kernel size for the prediction\nKERNEL_SIZE = 1000\n");
  fprintf(fp, "# Kernel size for the texture filter\nKERNEL_TEXT = 50\n\n");

  fprintf(fp, "# MISCELLANEOUS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Number of CPUs for parallel processing\nNUM_CPU = 1\n\n");

  fprintf(fp, "# OUTPUT\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Overwrite existing Improvements?\nOUTPUT_OVERWRITE = FALSE\n");
  fprintf(fp, "# Output format (ENVI or GTiff)\nOUTPUT_FORMAT = GTiff\n\n");

  fprintf(fp, "++PARAM_L2IMP_END++\n");

  fclose(fp);

  return SUCCESS;
}

