/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

This file is part of FORCE - Framework for Operational Radiometric 
Correction for Environmental monitoring.

Copyright (C) 2013-2018 David Frantz

FORCE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FORCE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FORCE.  If not, see <http://www.gnu.org/licenses/>.

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
This program generates a parameter file skeleton, which may be used to
parameterize the FORCE Level-2 Processing System
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/


#include "force.h"


int main( int argc, char *argv[] ){
FILE *fp;
char fname[MAX_STR_LEN];

  if (argc != 2){ printf("Usage: %s dir\n", argv[0]); exit(1);}

  sprintf(fname, "%s/%s", argv[1], "level2-skeleton.prm");
  if ((fp = fopen(fname, "w")) == NULL){
    printf("Unable to open output file!\n"); exit(1); }

  fprintf(fp, "++PARAM_LEVEL2_START++\n\n");

  fprintf(fp, "# DIRECTORIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# file queue\nFILE_QUEUE = NULL\n");
  fprintf(fp, "# output directory for Level 2 data\nDIR_LEVEL2 = NULL\n");
  fprintf(fp, "# temporary directory for decompressed data\nDIR_TEMP   = NULL\n");
  fprintf(fp, "# water vapor look-up-tables\nDIR_WVPLUT = NULL\n");
  fprintf(fp, "# aerosol optical depth look-up-tables\nDIR_AOD   = NULL\n");
  //fprintf(fp, "# dark object database\nDIR_DARKOB = NULL\n");
  fprintf(fp, "# DEM file\nFILE_DEM = NULL\n");
  fprintf(fp, "# tile white list\nFILE_TILE  = NULL\n\n");

  fprintf(fp, "# SPATIAL PROPERTIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# reproject and/or tile\nDO_REPROJ  = TRUE\nDO_TILE    = TRUE\n");
  fprintf(fp, "# tile size in target projection units\nTILE_SIZE  = 30000\n");
  fprintf(fp, "# output resolution in target projection units\nRESOLUTION = 30\n");
  fprintf(fp, "# origin of the tiles\nORIGIN_LAT = 60\nORIGIN_LON = -25\n");
  fprintf(fp, "# Target Projection in WKT, 2 lines!!!\nPROJECTION = \nPROJCS[\"ETRS89 / LAEA Europe\",GEOGCS[\"ETRS89\",DATUM[\"European_Terrestrial_Reference_System_1989\",SPHEROID[\"GRS 1980\",6378137,298.257222101,AUTHORITY[\"EPSG\",\"7019\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6258\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4258\"]],PROJECTION[\"Lambert_Azimuthal_Equal_Area\"],PARAMETER[\"latitude_of_center\",52],PARAMETER[\"longitude_of_center\",10],PARAMETER[\"false_easting\",4321000],PARAMETER[\"false_northing\",3210000],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AUTHORITY[\"EPSG\",\"3035\"]]\n");
  fprintf(fp, "# resampling option (one out of NN, BL, CC)\nRESAMPLING = CC\n");
  fprintf(fp, "# Resolution Merge (is ignored, if not Sentinel-2)\nRES_MERGE = IMPROPHE\n");
  fprintf(fp, "# Acceptable tier level\nTIER = 1\n\n");

  fprintf(fp, "# RADIOMETRIC CORRECTION OPTIONS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# apply topographic correction\nDO_TOPO = TRUE\n");
  fprintf(fp, "# apply atmospheric correction\nDO_ATMO = TRUE\n");
  fprintf(fp, "# apply BRDF correction\nDO_BRDF = TRUE\n");
  //fprintf(fp, "# use DODB\nDO_DODB = FALSE\n");
  fprintf(fp, "# image-based dark objects\nDO_AOD  = TRUE\n\n");

  fprintf(fp, "# ADVANCED RADIOMETRIC CORRECTION OPTIONS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "MULTI_SCATTERING = TRUE\n");
  fprintf(fp, "ADJACENCY_EFFECT = TRUE\n");
  fprintf(fp, "SUN_VIEW_GRID    = 5000\n");
  fprintf(fp, "WATER_VAPOR      = 1.0\n\n");

  fprintf(fp, "# CLOUD DETECTION OPTIONS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# max. cloud/shadow cover\nMAX_CLOUD_COVER_FRAME = 75\nMAX_CLOUD_COVER_TILE  = 75\n");
  fprintf(fp, "# Fmask thresholds\nCLOUD_THRESHOLD  = 0.225\nSHADOW_THRESHOLD = 0.02\n\n");

  fprintf(fp, "# OUTPUT OPTIONS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Output format (ENVI or GTiff)\nOUTPUT_FORMAT = GTiff\n");
  fprintf(fp, "# generate cloud distance output?\nOUTPUT_CLD     = TRUE\n");
  fprintf(fp, "# generate AOD@green output?\nOUTPUT_AOD     = TRUE\n");
  fprintf(fp, "# generate WVP?\nOUTPUT_WVP     = TRUE\n");
  fprintf(fp, "# generate view zenith output?\nOUTPUT_VZN = TRUE\n");
  fprintf(fp, "# generate HOT output?\nOUTPUT_HOT     = TRUE\n\n");

  fprintf(fp, "++PARAM_LEVEL2_END++\n");

  fclose(fp);

  return SUCCESS;
}

