/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

This file is part of FORCE - Framework for Operational Radiometric 
Correction for Environmental monitoring.

Copyright (C) 2013-2018 David Frantz

FORCE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FORCE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FORCE.  If not, see <http://www.gnu.org/licenses/>.

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
This program generates a parameter file skeleton, which may be used to
parameterize the FORCE Level-3 Processing System
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/


#include "force.h"


int main( int argc, char *argv[] ){
FILE *fp;
char fname[MAX_STR_LEN];

  if (argc != 2){ printf("usage: %s dir\n", argv[0]); exit(1);}

  sprintf(fname, "%s/%s", argv[1], "level3-skeleton.prm");
  if ((fp = fopen(fname, "w")) == NULL){
    printf("Unable to open output file!\n"); exit(1); }

  fprintf(fp, "++PARAM_LEVEL3_START++\n\n");

  fprintf(fp, "# DIRECTORIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Level-2 datapool (input)\nDIR_LEVEL2 = NULL\n");
  fprintf(fp, "# LSP datapool (input, optional)\nDIR_LSP = NULL\n");
  fprintf(fp, "# Level-3 datapool (output)\nDIR_LEVEL3 = NULL\n");
  fprintf(fp, "# White list of input/output tiles\nFILE_TILE  = NULL\n\n");

  fprintf(fp, "# SENSOR WHITE LIST\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Select observations from following sensors\n# See documentation for valid keys\n");
  fprintf(fp, "# Spectral decimation is taken care of\n");
  fprintf(fp, "SENSORS = LND04 LND05 LND07 LND08 SEN2A SEN2B\n\n");

  fprintf(fp, "# QAI SCREENING\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# This list controls, which QAI flags are masked out.\n# See documentation for valid keys\n");
  fprintf(fp, "SCREEN_QAI = NODATA CLOUD_OPAQUE CLOUD_BUFFER CLOUD_CIRRUS CLOUD_SHADOW SNOW SUBZERO SATURATION\n\n");

  fprintf(fp, "# SPATIAL PROPERTIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Composite extent in tile coordinates\n");
  fprintf(fp, "X_TILE_MIN = 0\n");
  fprintf(fp, "Y_TILE_MIN = 0\n");
  fprintf(fp, "X_TILE_MAX = 9999\n");
  fprintf(fp, "Y_TILE_MAX = 9999\n");
  fprintf(fp, "# Generate tiled output or compile a mosaic of composites\nDO_TILE = TRUE\n");
  fprintf(fp, "# Spatial resolution of composite\n# Image decimation/replication is taken care of\n");
  fprintf(fp, "RESOLUTION = 30\n");
  fprintf(fp, "# Use an approx. Point Spread Function when reducing spatial resolution\n");
  fprintf(fp, "REDUCE_PSF = FALSE\n");
  fprintf(fp, "# Use ImproPhe'd data at better spatial resolution?\nUSE_IMPROPHE = FALSE\n\n");

  fprintf(fp, "# INTER-ANNUAL SUITABILITY\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Target year\nYEAR_TARGET = 2010\n");
  fprintf(fp, "# +- years to be considered\nYEAR_NUM = 2\n");
  fprintf(fp, "# inter-/intra-annual tradeoff factor\nY_FACTOR = 0.75\n\n");

  fprintf(fp, "# INTRA-ANNUAL SUITABILITY\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Function values for the DOY scoring functions: s0, s1, s2\n");
  fprintf(fp, "# Gaussian or logistic S-shaped functions are available\n");
  fprintf(fp, "DOY_SCORE_0 = 0.01\n");
  fprintf(fp, "DOY_SCORE_1 = 1.00\n");
  fprintf(fp, "DOY_SCORE_2 = 0.01\n");
  fprintf(fp, "# DOY values for the DOY scoring functions: p0, p1, p2\n");
  fprintf(fp, "# They are not used in case of phenological normalization\n");
  fprintf(fp, "# However, they are used for the file name\n");
  fprintf(fp, "# p0 < p1 < p2 || if p0 > p1: p0 is from previous year || if p2 < p1: p2 is from next year\n");
  fprintf(fp, "DOY_STATIC_0 = 120\n");
  fprintf(fp, "DOY_STATIC_1 = 180\n");
  fprintf(fp, "DOY_STATIC_2 = 240\n");
  fprintf(fp, "# Select obs. that are not in seasonal window?\nOFF_SEASON = FALSE\n\n");

  fprintf(fp, "# PHENOLOGY-ADAPTIVE COMPOSITING\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# use LSP-adaptive algorithm?\nLSP_DO = FALSE\n");
  fprintf(fp, "LSP datasets | a unique pattern (substring of filename) must be given\n");
  fprintf(fp, "LSP_PATTERN_PAR_0  = POS\n");
  fprintf(fp, "LSP_PATTERN_PAR_1  = EOS\n");
  fprintf(fp, "LSP_PATTERN_PAR_2  = MOS\n");
  fprintf(fp, "# 1st year of LSP data\nLSP_1ST_YEAR = 2000\n");
  fprintf(fp, "# LSP-Values are reported in days, starting from this date in Y*365 + DOY\n");
  fprintf(fp, "LSP_START = 1\n");
  fprintf(fp, "# Use long-term average LSP if inter-annual variability exceeds this threshold (in days)\n");
  fprintf(fp, "LSP_THRESHOLD = 182\n");
  fprintf(fp, "# Nodata of LSP\nLSP_NODATA = -32767\n\n");

  fprintf(fp, "# COMPOSITING SCORES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Weighting scores. 0.0 disables the usage of a score.\n");
  fprintf(fp, "SCORE_DOY_WEIGHT    = 1.0\n");
  fprintf(fp, "SCORE_YEAR_WEIGHT   = 1.0\n");
  fprintf(fp, "SCORE_CLOUD_WEIGHT  = 0.4\n");
  fprintf(fp, "SCORE_HAZE_WEIGHT   = 0.3\n");
  fprintf(fp, "SCORE_CORREL_WEIGHT = 0.0\n");
  fprintf(fp, "SCORE_VZEN_WEIGHT   = 0.1\n");
  fprintf(fp, "# cloud distance after which sky is assumed to be clear (in m)\nDREQ = 3000\n");
  fprintf(fp, "# view zenith which gives score ~= 0 (in °)\nVREQ = 7.5\n\n");

  fprintf(fp, "# MISCELLANEOUS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Number of CPUs for parallel processing\nNUM_CPU = 1\n\n");

  fprintf(fp, "# OUTPUT\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Output format (ENVI or GTiff)\nOUTPUT_FORMAT = GTiff\n");
  fprintf(fp, "# Best available pixel composite\nOUTPUT_BAP = TRUE\n");
  fprintf(fp, "# Compositing information\nOUTPUT_INF = TRUE\n");
  fprintf(fp, "# Compositing scores\nOUTPUT_SCR = FALSE\n");
  fprintf(fp, "# Average of clear-sky obs.\nOUTPUT_AVG = FALSE\n");
  fprintf(fp, "# Standard dev. of clear-sky obs.\nOUTPUT_STD = FALSE\n");
  fprintf(fp, "# Minimum of clear-sky obs.\nOUTPUT_MIN = FALSE\n");
  fprintf(fp, "# Maximum of clear-sky obs.\nOUTPUT_MAX = FALSE\n");
  fprintf(fp, "# Range of clear-sky obs.\nOUTPUT_RNG = FALSE\n");
  fprintf(fp, "# Skewness of clear-sky obs.\nOUTPUT_SKW = FALSE\n");
  fprintf(fp, "# Kurtosis of clear-sky obs.\nOUTPUT_KRT = FALSE\n");
  fprintf(fp, "# 25%%-Quantiles of clear-sky obs.\nOUTPUT_Q25 = FALSE\n");
  fprintf(fp, "# 50%%-Quantiles of clear-sky obs.\nOUTPUT_Q50 = FALSE\n");
  fprintf(fp, "# 75%%-Quantiles of clear-sky obs.\nOUTPUT_Q75 = FALSE\n");
  fprintf(fp, "# Interquantile range of clear-sky obs.\nOUTPUT_IQR = FALSE\n\n");

  fprintf(fp, "++PARAM_LEVEL3_END++\n");

  fclose(fp);

  return SUCCESS;
}

