/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

This file is part of FORCE - Framework for Operational Radiometric 
Correction for Environmental monitoring.

Copyright (C) 2013-2018 David Frantz

FORCE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FORCE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FORCE.  If not, see <http://www.gnu.org/licenses/>.

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
This program generates a parameter file skeleton, which may be used to
parameterize the FORCE Time Series Analysis component
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/


#include "force.h"


int main( int argc, char *argv[] ){
FILE *fp;
char fname[MAX_STR_LEN];

  if (argc != 2){ printf("usage: %s dir\n", argv[0]); exit(1);}

  sprintf(fname, "%s/%s", argv[1], "tsa-skeleton.prm");
  if ((fp = fopen(fname, "w")) == NULL){
    printf("Unable to open output file!\n"); exit(1); }

  fprintf(fp, "++PARAM_TSA_START++\n\n");

  fprintf(fp, "# DIRECTORIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Level-2 datapool (input)\nDIR_LEVEL2 = NULL\n");
  fprintf(fp, "# Analysis Mask datapool (input, optional)\nDIR_MASK = NULL\n");
  fprintf(fp, "# Time Series Analysis datapool (output)\nDIR_TSA = NULL\n");
  fprintf(fp, "# White list of input/output tiles\nFILE_TILE  = NULL\n");
  fprintf(fp, "# Endmember file (only needed if INDEX = SMA\nFILE_ENDMEM  = NULL\n\n");

  fprintf(fp, "# SENSOR WHITE LIST\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Select observations from following sensors\n# See documentation for valid keys\n");
  fprintf(fp, "SENSORS = LND04 LND05 LND07 LND08 SEN2A SEN2B\n\n");

  fprintf(fp, "# QAI SCREENING\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# This list controls, which QAI flags are masked out.\n# See documentation for valid keys\n");
  fprintf(fp, "SCREEN_QAI = NODATA CLOUD_OPAQUE CLOUD_BUFFER CLOUD_CIRRUS CLOUD_SHADOW SNOW SUBZERO SATURATION\n\n");

  fprintf(fp, "# SPATIAL PROPERTIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Analysis extent in tile coordinates\n");
  fprintf(fp, "X_TILE_MIN = 0\n");
  fprintf(fp, "Y_TILE_MIN = 0\n");
  fprintf(fp, "X_TILE_MAX = 9999\n");
  fprintf(fp, "Y_TILE_MAX = 9999\n");
  fprintf(fp, "# Spatial resolution of composite\n# Image decimation/replication is taken care of\n");
  fprintf(fp, "RESOLUTION = 30\n");
  fprintf(fp, "# Use an approx. Point Spread Function when reducing spatial resolution\n");
  fprintf(fp, "REDUCE_PSF = FALSE\n");
  fprintf(fp, "# Use ImproPhe'd data at better spatial resolution?\nUSE_IMPROPHE = FALSE\n\n");

  fprintf(fp, "# TEMPORAL PROPERTIES\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# DOY range\nDOY_MIN = 1\nDOY_MAX = 366\n");
  fprintf(fp, "# Month range\nMONTH_MIN = 1\nMONTH_MAX = 12\n");
  fprintf(fp, "# Year range\nYEAR_MIN = 1900\nYEAR_MAX = 2100\n\n");

  fprintf(fp, "# TIME SERIES PARAMETERS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Spectral Index\n# See documentation for valid keys\nINDEX = NDVI\n");
  fprintf(fp, "# Center around pixel mean\nCENTER = FALSE\n");
  fprintf(fp, "# Standardize with pixel mean and standard deviation\nSTANDARD = FALSE\n");
  fprintf(fp, "# Fold by ...\n# See documentation for valid keys\nTREND = YEAR\n");
  fprintf(fp, "# Folding method\n# See documentation for valid keys\nFOLD = AVG\n");
  fprintf(fp, "# Tail-type for trend analysis\n# See documentation for valid keys\nTAIL = TWO\n");
  fprintf(fp, "# Confidence level for trend analysis\nCONF = 0.95\n");
  fprintf(fp, "# Filter spikes that are higher than this value\n");
  fprintf(fp, "# Values are in unscaled index units\n");
  fprintf(fp, "# Give 0 to disable despiking\nDESPIKE = 0\n\n");
  
  fprintf(fp, "# SPECTRAL UNMIXING PARAMETERS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# This block only applies if INDEX = SMA\n");
  fprintf(fp, "# Sum-to-One constrained unmixing?\nSMA_SUM_TO_ONE = TRUE\n");
  fprintf(fp, "# Non-negativity constrained unmixing?\nSMA_NON_NEG = TRUE\n");
  fprintf(fp, "# Apply shade normalization?\nSMA_SHD_NORM = TRUE\n");
  fprintf(fp, "# Endmember number\nSMA_ENDMEMBER = 1\n\n");

  fprintf(fp, "# INTERPOLATION PARAMETERS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Interpolation method\n# See documentation for valid keys\nINTERPOLATE = NONE\n");
  fprintf(fp, "# Max temporal distance for moving average filter\nMOVING_MAX = 16\n");
  fprintf(fp, "# Sigmas for RBF fits\nRBF_SIGMA = 8 16 32\n");
  fprintf(fp, "# Cutoff density for RBF fit\nRBF_CUTOFF = 0.95\n");
  fprintf(fp, "# Interpolate each n days\nINT_DAY = 16\n\n");

  fprintf(fp, "# PHENOLOGY PARAMETERS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Use data from this date (year-1)\nLSP_DOY_PREV_YEAR = 90\n");
  fprintf(fp, "# Until this date (year+1)\nLSP_DOY_NEXT_YEAR = 182\n");
  fprintf(fp, "# Seasonality is of Northern/Southern/Mixed type (0/1/2)\nLSP_HEMISPHERE = 0\n");
  fprintf(fp, "# Number of segments for the spline per year\nLSP_N_SEGMENT = 4\n\n");

  fprintf(fp, "# MISCELLANEOUS\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Number of CPUs for parallel processing\nNUM_CPU = 1\n\n");

  fprintf(fp, "# OUTPUT\n");
  fprintf(fp, "# ------------------------------------------------------------------------\n");
  fprintf(fp, "# Output format (ENVI or GTiff)\nOUTPUT_FORMAT = GTiff\n");
  fprintf(fp, "# Time Series Stack\nOUTPUT_TSS = FALSE\n");
  fprintf(fp, "# Model Residuals (SMA only)\nOUTPUT_RMS = FALSE\n");
  fprintf(fp, "# Basic Statistics\nOUTPUT_STA = TRUE\n");
  fprintf(fp, "# Time Series Interpolation\nOUTPUT_TSI = FALSE\n");
  fprintf(fp, "# Fold Time Series by Year\nOUTPUT_FBY = FALSE\n");
  fprintf(fp, "# Fold Time Series by Month\nOUTPUT_FBM = FALSE\n");
  fprintf(fp, "# Fold Time Series by Week\nOUTPUT_FBW = FALSE\n");
  fprintf(fp, "# Fold Time Series by Day\nOUTPUT_FBD = FALSE\n");
  fprintf(fp, "# Trend Analysis\nOUTPUT_TRD = FALSE\n");
  fprintf(fp, "# Change, Aftereffect, Trend Analysis\nOUTPUT_CAT = FALSE\n");
  //fprintf(fp, "# Time Series Fit Error Metrics\nOUTPUT_TSE = FALSE\n");
  //fprintf(fp, "# Time Series Fit Residuals\nOUTPUT_TSR = FALSE\n");
  fprintf(fp, "# LSP metrics (SPLITS is required)\nOUTPUT_LSP = FALSE\n\n");

  fprintf(fp, "++PARAM_TSA_END++\n");

  fclose(fp);

  return SUCCESS;
}

