/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

This file is part of FORCE - Framework for Operational Radiometric 
Correction for Environmental monitoring.

Copyright (C) 2013-2018 David Frantz

FORCE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FORCE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FORCE.  If not, see <http://www.gnu.org/licenses/>.

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
This program inflates QAI layers
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/


#include "force.h"


int main( int argc, char *argv[] ){
int2s *qai_bit;
int b = 0, nb = 12;
qai_t QAI;
double geotran[6];
char iname[MAX_STR_LEN];
char oname[MAX_STR_LEN];
char d_out[MAX_STR_LEN];
char *pch;
char *lock = NULL;
char **bn = NULL;
GDALDatasetH fp;
GDALRasterBandH band;
GDALDriverH driver;
char form[MAX_STR_LEN];
char ext[MAX_STR_LEN];
char **options = NULL;


  if (argc != 4){ printf("Usage: %s QAI dir format\n", argv[0]); exit(1);}

  // parse arguments
  strcpy(iname, argv[1]);
  strcpy(d_out, argv[2]);
  strcpy(form,  argv[3]);

  GDALAllRegister();
  CPLSetConfigOption("GDAL_PAM_ENABLED", "YES");

  // choose between formats
  if (strcmp(form, "ENVI") == 0){
    par.format = 0; strcpy(ext, "dat"); 
  } else if (strcmp(form, "GTiff") == 0){
    par.format = 1; strcpy(ext, "tif"); 
  } else {
    printf("unknown output format. "); return FAILURE;
  }
  driver = GDALGetDriverByName(form);

  // set GeoTiff options
  if (par.format == 1){
    options = CSLSetNameValue(options, "COMPRESS", "LZW");
    options = CSLSetNameValue(options, "PREDICTOR", "2");
    options = CSLSetNameValue(options, "INTERLEAVE", "BAND");
  }

  // get image info
  image_info_fullres(iname, geotran);
  par.res = geotran[1];

  // read QAI
  if (read_single_image_2s(iname, &qai_bit, 1, _NX_*par.res, _NY_*par.res, false) != SUCCESS){
    printf("could not inflate QAI.\n"); exit(1);}

  // inflate QAI
  alloc_qai(&QAI);
  parse_qai(QAI, qai_bit);

  // band names
  alloc_2D((void***)&bn, nb, MAX_STR_LEN, sizeof(char));
  sprintf(bn[0],  "valid data");           sprintf(bn[1],  "Cloud state");
  sprintf(bn[2],  "Cloud shadow flag");    sprintf(bn[3],  "Snow flag");
  sprintf(bn[4],  "Water flag");           sprintf(bn[5],  "Aerosol state");
  sprintf(bn[6],  "Subzero flag");         sprintf(bn[7],  "Saturation flag");
  sprintf(bn[8],  "High sun zenith flag"); sprintf(bn[9],  "Illumination state");
  sprintf(bn[10], "Slope flag");           sprintf(bn[11], "Water vapor flag");


  // output filename
  if (strstr(iname, "_LEVEL2_") != NULL){
    pch = strstr(iname, "QAI");
  } else if (strstr(iname, "_LEVEL3_") != NULL){
    pch = strstr(iname, "INF");
  } else {
    printf("unknown level.\n"); exit(1);
  }
  strncpy(pch, "QIM", 3);
  sprintf(oname, "%s/%s.%s", d_out, basename_without_ext(iname), ext);
  

  if ((lock = (char*)CPLLockFile(oname, 60)) == NULL){
    printf("Unable to lock file. "); return FAILURE;}

  // write masks
  if ((fp = GDALCreate(driver, oname, _NX_, _NY_, nb, GDT_Byte, options)) == NULL){
    printf("Error creating output file.\n"); exit(1);}

  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.off, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;

  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.cld, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;

  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.shd, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;
  
  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.snw, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;
  
  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.wtr, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;
  
  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.aod, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;
  
  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.sub, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;
  
  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.sat, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;
  
  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.sun, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;
  
  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.ill, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;
  
  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.slp, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;
  
  band = GDALGetRasterBand(fp, b+1);
  GDALRasterIO(band, GF_Write, 0, 0, _NX_, _NY_, QAI.wvp, _NX_, _NY_, GDT_Byte, 0, 0);
  if (bn != NULL && par.format == 0) GDALSetDescription(band, bn[b]);
  b++;

  // write geo-metadata
  GDALSetGeoTransform(fp, geotran);
  GDALSetProjection(fp, par.css_dst);

  // in case of ENVI, update description
  if (par.format == 0) GDALSetDescription(fp, "Quality Inflated Masks generated by FORCE");

  GDALClose(fp);

  // append/write metadata
  append_envi_header(oname, "Quality Inflated Masks generated by FORCE", "QIM", 0, _NX_, _NY_, nb, geotran, bn, NULL, NULL, NULL, 255, -1, -1, -1, -1, -1, -1, -1, NULL, NULL, NULL, NULL, -1, -1);

  CPLUnlockFile(lock);

  // clean
  free((void*)qai_bit);
  free_qai(QAI);
  
  if (bn != NULL){
    free_2D((void**)bn, nb);
    bn = NULL;
  }
  
  if (options != NULL){
     CSLDestroy(options); options = NULL;}

  return SUCCESS;
}

  