/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

This file is part of FORCE - Framework for Operational Radiometric 
Correction for Environmental monitoring.

Copyright (C) 2013-2018 David Frantz, Sebastian Mader

FORCE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FORCE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FORCE.  If not, see <http://www.gnu.org/licenses/>.

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

/** The queue implementation was gratefully provided by Sebastian Mader
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
This file contains an implementation of a First-In-First-Out queue
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/


#include "force.h"


/** Create FIFO
+++ This function creates a First-In-First-Out queue.
+++ Return: queue
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
queue_t* create_queue(){

  queue_t *q = (queue_t*) malloc(sizeof(queue_t));

  q->length = 0;
  q->first  = NULL;
  q->last   = NULL;

  return q;
}


/** Create FIFO
+++ This function destroys a First-In-First-Out queue.
+++ Return: void
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
void destroy_queue(queue_t* q){
node_t *el, *tmp;

  if (q->length != 0){

    el = q->first;

    while (el->next != NULL){
      tmp = el;
      el = el->next;
      free((void*)tmp);
    }

    free((void*)el);
  }

  free((void*)q);

  return;
}


/** Enqueue coordinate pair
+++ This function enqueues a coordinate pair to the end of a First-In-
+++ First-Out queue.
--- q:      FIFO queue (modified)
--- i:      line
--- j:      column
+++ Return: SUCCESS/FAILURE
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
int enqueue(queue_t* q, int i, int j){
node_t *el;

  if ((el = (node_t*) malloc(sizeof(node_t))) == NULL)
    return FAILURE;

  el->i = i;
  el->j = j;
  el->prev = NULL;
  el->next = NULL;

  if (q->length == 0){
    q->first = el;
    q->last = el;
  } else {
    q->last->next = el;
    el->prev = q->last;
    q->last = el;
  }

  q->length++;

  return SUCCESS;
}


/** Dequeue coordinate pair
+++ This function dequeues a coordinate pair at the beginning of a First-
+++ In-First-Out queue.
--- q:      FIFO queue (modified)
--- i:      line   (returned)
--- j:      column (returned)
+++ Return: SUCCESS/FAILURE
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
int dequeue(queue_t* q, int* i, int* j){
node_t *el;

  if (q->length == 0) return FAILURE;

  el = q->first;
  q->first = el->next;

  if (q->first == NULL){
    q->last = NULL;
  } else {
    el->next->prev = NULL;
  }

  *i = el->i;
  *j = el->j;

  free((void*)el);

  q->length--;

  return SUCCESS;
}

