/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

This file is part of FORCE - Framework for Operational Radiometric 
Correction for Environmental monitoring.

Copyright (C) 2013-2018 David Frantz

FORCE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FORCE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FORCE.  If not, see <http://www.gnu.org/licenses/>.

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

#include "force.h"


/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
This program returns the tile ID and pixel that contains the requested 
input coordinate.
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/


int main( int argc, char *argv[] ){
char *dir = NULL;
coord_t geo, map;
float startx, starty;
int idx, idy;

  if (argc != 5){ printf("usage: %s datacube lon lat res\n", argv[0]); exit(1);}

  // get command line parameters
  dir     = argv[1];
  geo.x   = atof(argv[2]);
  geo.y   = atof(argv[3]);
  par.res = atof(argv[4]);

  // read datacube definition
  if (read_datacube_def(dir) == FAILURE){
    printf("Reading datacube definition failed.\n"); exit(1);}

  // get target coordinates in target css coordinates
  if ((geo_to_map(geo.x,  geo.y, 
                  &map.x, &map.y, 
                   par.css_dst)) == FAILURE){
  printf("Computing target coordinates in dst_srs failed!\n"); exit(1);}


  // find the tile the target coordinates fall into
  tile_find(map.x, map.y, &startx, &starty, &idx, &idy);

  // Print to stdout
  printf("Point { LON/LAT (%.2f,%.2f) | X/Y (%.2f,%.2f) }\n  is in tile X%04d_Y%04d at pixel %d/%d\n", 
          geo.x, geo.y, map.x, map.y, idx, idy, (int)((map.x-startx)/par.res), (int)((starty-map.y)/par.res));

  exit(0);
}
