/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

This file is part of FORCE - Framework for Operational Radiometric 
Correction for Environmental monitoring.

Copyright (C) 2013-2018 David Frantz

FORCE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FORCE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FORCE.  If not, see <http://www.gnu.org/licenses/>.

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/

/**+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
This file contains some utility functions
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/


#include "force.h"


/** Print integer vector to stdout
--- v:      vector
--- name:   string that indicates what is printed (printed to stdout) 
--- n:      number of elements
--- big:    number of digits
+++ Return: void
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
void print_ivector(int *v,   char *name, int n, int big){
int i;

  printf("%s:", name);
  for (i=0; i<n; i++) printf(" %0*d", big, v[i]); printf("\n");

  return;
}


/** Print float vector to stdout
--- v:      vector
--- name:   string that indicates what is printed (printed to stdout) 
--- n:      number of elements
--- big:    number of digits before decimal point
--- small:  number of digits after decimal point
+++ Return: void
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
void print_fvector(float *v, char *name, int n, int big, int small){
int i;

  printf("%s:", name);
  for (i=0; i<n; i++) printf(" %0*.*f", big+small+1, small, v[i]); printf("\n");

  return;
}


/** Print double vector to stdout
--- v:      vector
--- name:   string that indicates what is printed (printed to stdout) 
--- n:      number of elements
--- big:    number of digits before decimal point
--- small:  number of digits after decimal point
+++ Return: void
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
void print_dvector(double *v, char *name, int n, int big, int small){
int i;

  printf("%s:", name);
  for (i=0; i<n; i++) printf(" %0*.*f", big+small+1, small, v[i]); printf("\n");

  return;
}


/** Convert radian angle to degrees
--- x:      angle in radian
+++ Return: angle in degree
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
float rad2deg(float x){

  return (x*_R2D_CONV_);
}


/** Convert degree angle to radians
--- x:      angle in degree
+++ Return: angle in radian
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
float deg2rad(float x){

  return (x*_D2R_CONV_);
}


/** Measure time
+++ This function measures the processing time and prints to stdout
--- start:  start time
+++ Return: time in seconds
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
double proctime(time_t start){
time_t now;
double secs;

  time(&now); secs = difftime(now, start);

  return secs;
}


/** Measure time and print
+++ This function measures the processing time and prints to stdout
--- string: string that indicates what was measured (printed to stdout) 
--- start:  start time
+++ Return: void
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
void proctime_print(char *string, time_t start){
time_t now;
double secs;
int mins;

  time(&now); secs = difftime(now, start);
  if (secs >= 60){
    mins = floor(secs/60); secs = secs-mins*60;
  } else mins = 0;
  printf("%s: %02d mins %02.0f secs\n", string, mins, secs);

  return;
}


/** Image coordinate to coarse grid coordinate
--- x:      image coordinate (x = column or y = row)
+++ Return: coarse grid coordinate
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++**/
int tocg(int x){

  return(floor(x/atc.gres));
}

